@@ -21,6 +21,11 @@ BED15File <- function(resource) {
   new("BED15File", resource = resource)
 }
 
+setClass("BEDPEFile", contains = "BEDFile")
+BEDPEFile <- function(resource) {
+    new("BEDPEFile", resource = resource)
+}
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Export
 ###
@@ -102,7 +107,7 @@ setMethod("export", c("GenomicRanges", "BEDFile"),
                 if (any(df[[2]] + lastSize + lastStart != df[[3]]) ||
                     any(sub(",.*", "", blockStarts) != "0"))
                   stop("blocks must span entire feature")
-                blockCount <- elementLengths(object$blocks)
+                blockCount <- elementNROWS(object$blocks)
                 if (!is.null(object$blockCount))
                   if (!identical(blockCount, as.integer(object$blockCount)))
                     stop("incorrect block counts given block sizes")
@@ -245,20 +250,8 @@ setMethod("import", "BEDFile",
                                  extraCols = extraCols)
               return(ans)
             }
-            if (is(file, "BEDGraphFile")) {
-              bedClasses <- c("character", "integer", "integer", "numeric")
-              bedNames <- c("chrom", "start", "end", "score")
-            } else {
-              bedNames <- c("chrom", "start", "end", "name",
-                            "score", "strand", "thickStart",
-                            "thickEnd", "itemRgb", "blockCount",
-                            "blockSizes", "blockStarts")
-              bedClasses <- c("character", "integer", "integer", "character",
-                              "numeric", "character", "integer", "integer",
-                              "character", "integer", "character", "character")
-            }
-            if (is(file, "BED15File"))
-              bedNames <- c(bedNames, "expCount", "expIds", "expScores")
+            bedClasses <- colClasses(file)
+            bedNames <- names(bedClasses)
             normArgColnames <- function(validNames) {
               if (is.null(colnames))
                 colnames <- validNames
@@ -292,7 +285,7 @@ setMethod("import", "BEDFile",
               `tail<-` <- function(x, n, value)
                 if (n != 0) c(head(x, -n), value) else x
               pushBack(line, con)
-              colsInFile <- seq_len(length(strsplit(line, "[\t ]")[[1]]))
+              colsInFile <- seq_len(length(strsplit(line, "\\s+")[[1]]))
               presentNames <- bedNames[colsInFile]
               tail(presentNames, length(extraCols)) <- names(extraCols)
               bedNames <- presentNames
@@ -348,6 +341,16 @@ setMethod("import", "BEDFile",
                         which = which)
           })
 
+setGeneric("colClasses", function(x) standardGeneric("colClasses"))
+
+setMethod("colClasses", "BEDFile", function(x) {
+              c(chrom="character", start="integer", end="integer",
+                name="character", score="numeric", strand="character",
+                thickStart="integer", thickEnd="integer",
+                itemRgb="character", blockCount="integer",
+                blockSizes="character", blockStarts="character")
+          })
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### BED15 (Microarray) Support
 ###
@@ -453,6 +456,11 @@ setAs("character", "Bed15TrackLine",
 
 setMethod("fileFormat", "Bed15TrackLine", function(x) "bed15")
 
+setMethod("colClasses", "BED15File", function(x) {
+              c(callNextMethod(), expCount="integer", expIds="character",
+                expScores="character")
+          })
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### bedGraph (formerly subset of WIG) support
 ###
@@ -477,6 +485,99 @@ setMethod("export.bedGraph", "ANY",
             export(object, con, "bedGraph", ...)
           })
 
+setMethod("colClasses", "BEDGraphFile", function(x) {
+              c(chrom="character", start="integer", end="integer",
+                score="numeric")
+          })
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### BEDPE (bedtools paired end) support
+###
+
+setMethod("import", "BEDPEFile",
+          function(con, format, text, ...) {
+              gr <- callNextMethod()
+              df <- mcols(gr)
+              mcols(gr) <- NULL
+              df$strand2[is.na(df$strand2)] <- "*"
+              gr2 <- with(df, GRanges(chrom2, IRanges(start2+1L, end2),
+                                      strand2, seqinfo=seqinfo(gr)))
+              df <- subset(df, select=-(chrom2:end2))
+              df$strand2 <- NULL
+              Pairs(gr, gr2, df)
+          })
+
+setMethod("colClasses", "BEDPEFile", function(x) {
+              c(chrom="character", start="integer", end="integer",
+                chrom2="character", start2="integer", end2="integer",
+                name="character", score="numeric", strand="character",
+                strand2="character")
+          })
+
+setMethod("export", c("ANY", "BEDPEFile"),
+          function(object, con, format, ...)
+          {
+            if (!missing(format))
+              checkArgFormat(con, format)
+            cl <- class(object)
+            track <- try(as(object, "Pairs"), silent = TRUE)
+            if (class(track) == "try-error") {
+              stop("cannot export object of class '", cl, "': ", track)
+            }
+            export(track, con, ...)
+          })
+
+setMethod("export", c("Pairs", "BEDPEFile"),
+          function(object, con, format, append = FALSE, ignore.strand = FALSE)
+          {
+            if (!missing(format))
+              checkArgFormat(con, format)
+            df <- data.frame(
+                             seqnames(first(object)),
+                             start(first(object)) - 1L,
+                             end(first(object)),
+                             seqnames(second(object)),
+                             start(second(object)) - 1L,
+                             end(second(object))
+                            )
+            score <- mcols(object)$score
+            if (!is.null(score)) {
+              if (!is.numeric(score) || any(is.na(score)))
+                stop("Scores must be non-NA numeric values")
+            } else {
+                score <- 0
+            }
+            name <- mcols(object)$name
+            if (is.null(name))
+              name <- names(object)
+            if (is.null(name))
+              name <- rep(NA, length(object))
+            df$name <- name
+            df$score <- score
+            if (ignore.strand) {
+                strand1 <- NA # strand cannot be null in bedpe
+                strand2 <- NA
+            } else {
+                strand1 <- strand(first(object))
+                strand2 <- strand(second(object))
+                strand1[strand1 == "*"] <- NA
+                strand2[strand2 == "*"] <- NA
+            }
+            df$strand1 <- strand1
+            df$strand2 <- strand2
+            mcol_names <- setdiff(names(mcols(object)), c("name", "score"))
+            df <- cbind(df, mcols(object)[,mcol_names])
+            scipen <- getOption("scipen")
+            options(scipen = 100) # prevent use of scientific notation
+            on.exit(options(scipen = scipen))
+            file <- con
+            con <- connection(con, if (append) "a" else "w")
+            write.table(df, con, sep = "\t", col.names = FALSE,
+                        row.names = FALSE, quote = FALSE, na = ".")
+            release(con)
+            invisible(file)
+          })
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion
 ###
@@ -485,14 +586,14 @@ setGeneric("asBED", function(x, ...) standardGeneric("asBED"))
 
 setMethod("asBED", "GRangesList", function(x) {
   x_range <- range(x)
-  if (any(elementLengths(x_range) != 1L))
+  if (any(elementNROWS(x_range) != 1L))
     stop("Empty or multi-strand/seqname elements not supported by BED")
   gr <- unlist(x_range, use.names=FALSE)
   values(gr) <- values(x)
   values(gr)$name <- names(x)
   x_ranges <- ranges(unlist(x, use.names=FALSE))
   ord_start <- order(start(x_ranges))
-  x_ranges <- shift(x_ranges, 1L - rep(start(gr), elementLengths(x)))[ord_start]
+  x_ranges <- shift(x_ranges, 1L - rep(start(gr), elementNROWS(x)))[ord_start]
   values(gr)$blocks <- split(x_ranges, togroup(x)[ord_start])
   gr
 })
@@ -503,16 +604,10 @@ setMethod("asBED", "GRangesList", function(x) {
 
 setGeneric("blocks", function(x, ...) standardGeneric("blocks"))
 
-setMethod("blocks", "RangedData",
-          function(x)
-          {
-            blocks(as(x, "GRanges"))
-          })
-
 setMethod("blocks", "GenomicRanges",
           function(x)
           {
-            block_counts <- elementLengths(values(x)$blocks)
+            block_counts <- elementNROWS(values(x)$blocks)
             gr <- GRanges(rep(seqnames(x), block_counts),
                           shift(unlist(values(x)$blocks, use.names = FALSE),
                                 rep(start(x), block_counts) - 1L),
