@@ -21,7 +21,7 @@ setMethod("seqinfo", "BigWigFile", function(x) {
   Seqinfo(names(seqlengths), seqlengths) # no circularity information
 })
 
-setClass("BigWigFileList", contains = "SimpleList",
+setClass("BigWigFileList", contains = "RTLFileList",
     prototype = prototype(elementType = "BigWigFile"))
 
 BigWigFileList <- function(path)
@@ -100,7 +100,7 @@ setMethod("export", c("GenomicRanges", "BigWigFile"),
           function(object, con, format,
                    dataFormat = c("auto", "variableStep", "fixedStep",
                                   "bedGraph"),
-                   compress = TRUE)
+                   compress = TRUE, fixedSummaries = FALSE)
           {
             if (!missing(format))
               checkArgFormat(con, format)
@@ -139,18 +139,19 @@ setMethod("export", c("GenomicRanges", "BigWigFile"),
                             writer = .bigWigWriter, trackLine = FALSE)
             storage.mode(seqlengths) <- "integer"
             invisible(BigWigFile(.Call(BWGSectionList_write, sectionPtr,
-                                       seqlengths, compress, con)))
+                                       seqlengths, compress, fixedSummaries,
+                                       con)))
           })
 
 setMethod("export", c("List", "BigWigFile"),
-          function(object, con, format, compress = TRUE)
+          function(object, con, format, compress = TRUE, fixedSummaries = FALSE)
           {
             if (!missing(format))
               checkArgFormat(con, format)
             con <- path.expand(path(con))
             if (!isTRUEorFALSE(compress))
               stop("'compress' must be TRUE or FALSE")
-            seqlengths <- elementLengths(object)
+            seqlengths <- elementNROWS(object)
             sectionPtr <- NULL # keep adding to the same linked list
             on.exit(.Call(BWGSectionList_cleanup, sectionPtr))
             writer <- BigWigWriter(object)
@@ -158,7 +159,8 @@ setMethod("export", c("List", "BigWigFile"),
               sectionPtr <- writer(chr, sectionPtr)
             }
             invisible(BigWigFile(.Call(BWGSectionList_write, sectionPtr,
-                                       seqlengths, compress, con)))
+                                       seqlengths, compress, fixedSummaries,
+                                       con)))
           })
 
 setGeneric("BigWigWriter", function(x) standardGeneric("BigWigWriter"))
@@ -191,23 +193,14 @@ setMethod("import.bw", "ANY",
             import(con, "BigWig", ...)
           })
 
-
-### FIXME: For chr20, WGS coverage, coercion RangedData->GRanges takes
-### 2X as long as reading. The most common use case is reading long
-### vectors as Rles. Constructing an RleList is complicated by
-### 'which'. As long as 'which' spans chromosomes, we can quickly
-### coerce the GRanges (or faster yet, a RangedData) to an
-### RleList. Consumes memory, but time cost is minimal, when 'which'
-### is trivial (an entire genome or chromosome).
-
 ### Consider the use cases:
 ### - Extract whole genome or whole chromosome coverage as Rle (ChIP-seq)
-###   - Use rdToRle() fast path
+###   - Use coverage() fast path
 ### - Extract single position coverage for millions of variants
 ###   - Slow to query by position, so extract whole genome/chromosome (above),
 ###     then use findRun trick.
 ### - Extract single position coverage for a hundred hot-spot variants
-###   - rdToRle() should be fast enough, followed by findRun trick.
+###   - coverage() should be fast enough, followed by findRun trick.
 ### - Extract coverage for one/all genes and summarize (RNA-seq?)
 ###   - probably want summarize,BigWigFile
 
@@ -242,32 +235,28 @@ setMethod("import", "BigWigFile",
             if (as != "NumericList") {
               which <- as(which, "NormalIRangesList")
             }
-            rd <- .Call(BWGFile_query, path.expand(path(con)),
-                        as.list(which),
-                        identical(colnames(selection), "score"), 
-                        as == "NumericList")
+            C_ans <- .Call(BWGFile_query, path.expand(path(con)),
+                           as.list(which),
+                           identical(colnames(selection), "score"), 
+                           as == "NumericList")
             if (as == "NumericList") {
-                rd <- as(rd, "NumericList")
-                names(rd) <- rep(names(which), elementLengths(which))
-                metadata(rd) <- list(ranges = as(which, "GRanges"))
-                rd
+              ans <- as(C_ans, "NumericList")
+              names(ans) <- rep(names(which), elementNROWS(which))
+              metadata(ans) <- list(ranges = as(which, "GRanges"))
+              ans
             } else {
-              seqinfo(rd) <- si
+              gr <- as(C_ans[[1L]], "GRanges")
+              mcols(gr) <- unlist(C_ans[[2L]], use.names=FALSE)
+              seqinfo(gr) <- si
               if (as == "RleList") {
-                rdToRle(rd)
-              } else if (as == "GRanges") {
-                strand(rd) <- "*"
-                rd <- as(rd, "GRanges")
-              } else rd
+                coverage(gr, weight="score")
+              } else {
+                strand(gr) <- "*"
+                gr
+              }
             }
            })
 
-rdToRle <- function(x) {
-  RleList(mapply(function(r, v, sl) {
-    coverage(r, width=sl, weight=v$score)
-  }, ranges(x), values(x), seqlengths(x), SIMPLIFY=FALSE), compress=FALSE)
-}
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Summary
 ###
@@ -309,7 +298,8 @@ setMethod("summary", "BigWigFile",
 
 wigToBigWig <-
   function(x, seqinfo,
-           dest = paste(file_path_sans_ext(x, TRUE), "bw", sep = "."))
+           dest = paste(file_path_sans_ext(x, TRUE), "bw", sep = "."),
+           clip = FALSE)
   {
     if (!isSingleString(x))
       stop("'x' must be a single string, the path to a WIG file")
@@ -317,11 +307,28 @@ wigToBigWig <-
       stop("'dest' must be a single string, the path to the BigWig output")
     if (!is(seqinfo, "Seqinfo"))
       stop("'seqinfo' must be NULL or a Seqinfo object")
+    if (!isTRUEorFALSE(clip))
+      stop("'clip' must be TRUE or FALSE")
     seqlengths <- seqlengths(seqinfo)
     if (any(is.na(seqlengths)))
       stop("'seqlengths(seqinfo)' must not contain any 'NA' values")
     x <- path.expand(x)
     dest <- path.expand(dest)
-    ans <- .Call(BWGFile_fromWIG, x, seqlengths, dest)
+    ans <- .Call(BWGFile_fromWIG, x, clip, seqlengths, dest)
     invisible(BigWigFile(ans))
   }
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Utilities
+###
+
+## Remote data are cached locally. Need a way to cleanup.
+
+cleanupBigWigCache <- function(maxDays = 0) {
+  stopifnot(isSingleNumber(maxDays))
+  dir <- "/tmp/udcCache"
+  if (file.exists(dir)) {
+      invisible(.Call(R_udcCleanup, maxDays))
+  }
+}
+
