@@ -90,7 +90,7 @@ test_bed <- function() {
   test <- import(test_bed, colnames = subcols)
   checkIdentical(correct_subcols, test)
   
-  which <- RangesList(chr7 = ranges(correct_gr)[1:2])
+  which <- correct_gr[1:2]
   correct_which <- subsetByOverlaps(correct_ucsc, which)
   test <- import(test_bed, which = which)
   checkIdentical(correct_which, test)
@@ -288,3 +288,40 @@ test_extendedBed <- function()
 
     
 } 
+
+test_bedpe <- function() {
+    path <- system.file("tests", "test.bedpe", package="rtracklayer")
+    nms <- c("TUPAC_0001:3:1:0:1452#0", "TUPAC_0001:3:1:0:1472#0",
+             "TUPAC_0001:3:1:1:1833#0")
+    gr1 <- GRanges(c("chr7", "chr11", "chr20"),
+                   IRanges(c(118965073, 46765607, 54704675),
+                           c(118965122, 46765656, 54704724)),
+                   strand="+")
+    gr2 <- GRanges(c("chr7", "chr11", "chr20"),
+                   IRanges(c(118970080, 46769935, 54708988),
+                           c(118970129, 46769984, 54709037)),
+                   strand="-")
+    pairs <- Pairs(gr1, gr2, name=nms, score=37)
+    bedpe <- import(path)
+
+    checkIdentical(pairs, bedpe)
+
+    # test export
+    test_bedpe_out <- file.path(tempdir(), "test.bedpe")
+    on.exit(unlink(test_bedpe_out))
+    export(bedpe, test_bedpe_out)
+    test <- import(test_bedpe_out)
+    checkIdentical(bedpe, test)
+
+    bedpe2 <- bedpe
+    mcols(bedpe2)$qvalue <- c(0.02, 0.03, 0.05)
+    mcols(bedpe2)$annotation <- c("promoter", "enhancer", "intron")
+
+    # test extended bedpe
+    test_bedpe2_out <- file.path(tempdir(), "test2.bedpe")
+    on.exit(unlink(test_bedpe2_out))
+    export(bedpe2, test_bedpe2_out)
+    test <- import(test_bedpe2_out,
+                   extraCols=c(qvalue="numeric", annotation="character"))
+    checkIdentical(bedpe2, test)
+}
