@@ -94,8 +94,8 @@ createBWGSection_Atomic(const char *seq, int start, double *score,
   return section;
 }
 
-static int itemsPerSlot = 512;
-static int blockSize = 1024;
+static int itemsPerSlot = 1024;
+static int blockSize = 256;
 
 static void BWGSectionList_addRle(struct bwgSection **sections, const char *seq,
                                   SEXP r_ranges, double *score,
@@ -179,7 +179,7 @@ static struct hash *createIntHash(SEXP v) {
 
 /* --- .Call ENTRY POINT --- */
 SEXP BWGSectionList_write(SEXP r_sections, SEXP r_seqlengths, SEXP r_compress,
-                          SEXP r_file)
+                          SEXP r_fixed_summaries, SEXP r_file)
 {
   struct bwgSection *sections = NULL;
   struct hash *lenHash = createIntHash(r_seqlengths);
@@ -189,7 +189,7 @@ SEXP BWGSectionList_write(SEXP r_sections, SEXP r_seqlengths, SEXP r_compress,
   }
   pushRHandlers();
   bwgCreate(sections, lenHash, blockSize, itemsPerSlot, asLogical(r_compress),
-            FALSE /*keepAllChromosomes*/, TRUE /*fixedSummaries*/,
+            FALSE /*keepAllChromosomes*/, asLogical(r_fixed_summaries),
             (char *)CHAR(asChar(r_file)));
   freeHash(&lenHash);
   popRHandlers();
@@ -293,7 +293,7 @@ SEXP BWGFile_query(SEXP r_filename, SEXP r_ranges, SEXP r_return_score,
       hits = slCat(queryHits, hits);
     } 
 
-    /* RangedData */
+    /* GRanges */
     if (!return_list) {
       int nhits = slCount(hits);
       slReverse(&hits);
@@ -333,8 +333,10 @@ SEXP BWGFile_query(SEXP r_filename, SEXP r_ranges, SEXP r_return_score,
     PROTECT(dataFrameList =
             new_SimpleList("SimpleSplitDataFrameList", dataFrameListEls));
     PROTECT(rangesList = new_SimpleList("SimpleRangesList", rangesListEls));
-    ans = new_RangedData("RangedData", rangesList, dataFrameList);
-    UNPROTECT(4);
+    PROTECT(ans = allocVector(VECSXP, 2));
+    SET_ELEMENT(ans, 0, rangesList);
+    SET_ELEMENT(ans, 1, dataFrameList);
+    UNPROTECT(5);
   }
 
   lmCleanup(&lm);
@@ -379,17 +381,26 @@ SEXP BWGFile_summary(SEXP r_filename, SEXP r_chrom, SEXP r_ranges,
 
 #include "ucsc/verbose.h"
 
-SEXP BWGFile_fromWIG(SEXP r_infile, SEXP r_seqlengths, SEXP r_outfile) {
+SEXP BWGFile_fromWIG(SEXP r_infile, SEXP r_clip, SEXP r_seqlengths,
+		     SEXP r_outfile)
+{
   pushRHandlers();
   struct lm *lm = lmInit(0);
   struct hash *lenHash = createIntHash(r_seqlengths);
+  Rboolean clip = asLogical(r_clip);
   struct bwgSection *sections =
-    bwgParseWig((char *)CHAR(asChar(r_infile)), FALSE, lenHash, itemsPerSlot,
+    bwgParseWig((char *)CHAR(asChar(r_infile)), clip, lenHash, itemsPerSlot,
                 lm);
-  bwgCreate(sections, lenHash, blockSize, itemsPerSlot, TRUE, TRUE, TRUE,
+  bwgCreate(sections, lenHash, blockSize, itemsPerSlot, TRUE, TRUE, FALSE,
             (char *)CHAR(asChar(r_outfile)));
   lmCleanup(&lm);
   freeHash(&lenHash);
   popRHandlers();
   return r_outfile;
 }
+
+SEXP R_udcCleanup(SEXP r_maxDays) {
+    double maxDays = asReal(r_maxDays);
+    bits64 size = udcCleanup(udcDefaultDir(), maxDays, FALSE);
+    return ScalarReal(size);
+}
